# Installation instructions #

# Dependencies #

SyB3R depends on a number of external software packages, some of which are optional depending on which parts of SyB3R are built.
    * cmake (>= 2.8)
    * C++ compiler (tested on gcc with make)
    * Boost
    * OpenEXR
    * ImageMagick
    * Eigen
    * TinyXML2
    * Blender + Cycles

Optional:
    * Exiv2
    * PCL (>= 1.7)
    * Doxygen (for building the documentation)

# Building #

SyB3R uses cmake to configure and build the build system files. On linux the preferred method are makefiles:

    cd [path/to/syb3r/]
    mkdir buildRelease
    cd buildRelease
    cmake .. -DCMAKE_BUILD_TYPE=Release -Dsyb3r_build_analysis=ON -Dsyb3r_build_tools=ON -Dsyb3r_build_examples=ON
    make -j

This will build the enabled libraries and executables in the buildRelease/ subfolder.
The following cmake options are available:

|Option                                         | Default | Meaning                                                                       |
|-----------------------------------------------|---------|-------------------------------------------------------------------------------|
|syb3r_build_analysis                           | On      | Build the library containing the evaluation code (requires PCL)               |
|syb3r_build_tools                              | On      | Build tools (executables)                                                     |
|syb3r_build_examples                           | Off     | Build samples (executables)                                                   |
|                                               |         |                                                                               |
|syb3r_has_exiv2                                | On      | Build stuff that depend on libexiv2 (e.g., camera response curve estimation)  |


We tested compilation on the following systems:

    * Fedora 23
    * Ubuntu 14.04

The libraries can be used in an external cmake project by importing the respective targets in the cmake file, e.g.

    include([path/to/syb3r/]buildRelease/libs/common/syb3rCommon-exports.cmake)
    include([path/to/syb3r/]buildRelease/libs/Analysis/syb3rAnalysis-exports.cmake)
    include([path/to/syb3r/]buildRelease/libs/ReconstructionInvocation/syb3rReconstructionInvocation-exports.cmake)
    include([path/to/syb3r/]buildRelease/libs/Synthesis/syb3rSynthesis-exports.cmake)

or whatever subset is needed.

To build the documentation, simply execute \[path/to/syb3r/\]doc/buildDoc.sh and open \[path/to/syb3r\]/doc/html/index.html.

