
#include <syb3r/data/PlyPointcloudFile.h>
#include <syb3r/data/Pointcloud.h>

#include <boost/optional.hpp>
#include <boost/lexical_cast.hpp>

#include <thread>
#include <iostream>
#include <fstream>



inline Eigen::Vector3f colorRamp(float f) {
    Eigen::Vector3f color;
    if (f < 0.0f) {
        color[0] = 0.0f;
        color[1] = 0.0f;
        color[2] = 1.0f;
        return color;
    }
    if (f < 0.25f) {
        color[0] = 0.0f;
        color[1] = f / 0.25f;
        color[2] = 1.0f;
        return color;
    }
    if (f < 0.5f) {
        color[0] = 0.0f;
        color[1] = 1.0f;
        color[2] = (0.5f-f) / 0.25f;
        return color;
    }
    if (f < 0.75f) {
        color[0] = (f-0.5f) / 0.25f;
        color[1] = 1.0f;
        color[2] = 0.0f;
        return color;
    }
    if (f < 1.0f) {
        color[0] = 1.0f;
        color[1] = (1.0f-f) / 0.25f;
        color[2] = 0.0f;
        return color;
    }
    color[0] = 1.0f;
    color[1] = 0.0f;
    color[2] = 0.0f;

    return color;
}

int main(int argc, const char** argv)
{
    boost::optional<std::string> inputFilename;
    boost::optional<std::string> outputFilename;

    boost::optional<float> maxDistance;

    for (unsigned i = 1; i < argc; i++) {
        std::string arg(argv[i]);
        if (arg == "--input") {
            if (i+1 >= argc) {
                std::cerr << "Missing argument!" << std::endl;
                exit(1);
            }
            inputFilename = argv[i+1];
            i++;
        } else
        if (arg == "--output") {
            if (i+1 >= argc) {
                std::cerr << "Missing argument!" << std::endl;
                exit(1);
            }
            outputFilename = argv[i+1];
            i++;
        } else
        if (arg == "--maxDistance") {
            if (i+1 >= argc) {
                std::cerr << "Missing argument!" << std::endl;
                exit(1);
            }
            try {
                maxDistance = boost::lexical_cast<float>(argv[i+1]);
            } catch (...) {
                std::cerr << "Invalid argument!" << std::endl;
                exit(1);
            }
            i++;
        } else {
            std::cerr << "Unknown argument: " << arg << std::endl;
            return 1;
        }
    }

    if (!inputFilename) {
        std::cerr << "Input filename (--input) mandatory!"<<std::endl;
        exit(1);
    }
    if (!outputFilename) {
        std::cerr << "Output filename (--output) mandatory!"<<std::endl;
        exit(1);
    }
    if (!maxDistance) {
        std::cerr << "Max distance (--maxDistance) mandatory!"<<std::endl;
        exit(1);
    }

    syb3r::data::Pointcloud pcld;

    std::vector<float> vertexQuality;

    syb3r::data::importPLYPointcloudFile(pcld, vertexQuality, *inputFilename);

    if (vertexQuality.empty()) {
        std::cerr << "Input pointcloud ply file needs an attribute 'quality' that stores the distance!"<<std::endl;
        exit(1);
    }

    syb3r::data::writePLYPointcloudFile(*outputFilename, pcld.getCount(), [&](unsigned idx, Eigen::Vector3f &position, Eigen::Vector3f &normal, Eigen::Vector3f &color){
        position = pcld.getPosition(idx);
        normal = pcld.getNormal(idx);
        color = colorRamp(vertexQuality[idx] / *maxDistance);
    });

}