
#include <syb3r/data/Dataset.h>
#include <syb3r/data/PlyPointcloudFile.h>

#include <syb3r/tools/TaskScheduler.h>

#include <boost/lexical_cast.hpp>

#include <thread>

#include <iostream>

#include <fstream>

void printUsageAndExit()
{
    std::cout << "usage: --dataset path/to/dataset/scene.xml --output destination/filename.ply [--subsampling subsamplingAmount]" << std::endl;
    std::cout << "The subsampling amount defaults to 4 if not specified" << std::endl;
    exit(0);
}

int main(int argc, const char **argv)
{
    syb3r::tools::TaskScheduler::Init(std::thread::hardware_concurrency());


    boost::optional<std::string> datasetFilename;
    boost::optional<std::string> outputFilename;
    unsigned subsampling = 4;

    for (unsigned i = 1; i < argc; i++) {
        const std::string argument(argv[i]);
        if (argument == "--dataset") {
            if (i+1 >= argc) {
                std::cout << "Missing argument!" << std::endl;
                exit(1);
            }
            datasetFilename = argv[i+1];
            i++;
        } else
        if (argument == "--output") {
            if (i+1 >= argc) {
                std::cout << "Missing argument!" << std::endl;
                exit(1);
            }
            outputFilename = argv[i+1];
            i++;
        } else
        if (argument == "--subsampling") {
            if (i+1 >= argc) {
                std::cout << "Missing argument!" << std::endl;
                exit(1);
            }
            try {
                subsampling = boost::lexical_cast<unsigned>(argv[i+1]);
            } catch (...) {
                std::cout << "Invalid argument!" << std::endl;
                exit(1);
            }
            i++;
        } else {
            std::cout << "Unknown argument: '" << argument << "'"<< std::endl;
            printUsageAndExit();
        }
    }

    if (!datasetFilename) {
        std::cout << "Input dataset is mandatory!" << std::endl;
        printUsageAndExit();
    }

    if (!outputFilename) {
        std::cout << "Output filename is mandatory!" << std::endl;
        printUsageAndExit();
    }

    std::cout << "Loading scene" << std::endl;
    syb3r::data::Dataset referenceDataset;
    referenceDataset.loadFromXML(*datasetFilename);

    std::vector<Eigen::Vector3f> refPcld;
    std::vector<unsigned> objId;
    referenceDataset.calcPointCloudFromDepth(refPcld, objId, subsampling);

    {
        std::fstream file;
        file.exceptions(std::ifstream::failbit | std::ifstream::badbit);
        file.open(*outputFilename, std::fstream::out);

        file
            << "ply" << std::endl
            << "format ascii 1.0" << std::endl
            << "element vertex " << refPcld.size() << std::endl
            << "property float x" << std::endl
            << "property float y" << std::endl
            << "property float z" << std::endl
            << "property int objId" << std::endl
            << "end_header" << std::endl;

        for (unsigned i = 0; i < refPcld.size(); i++) {
            file
                << refPcld[i][0] << " "
                << refPcld[i][1] << " "
                << refPcld[i][2] << " "
                << (unsigned)objId[i] << " "
                << std::endl;
        }
    }
}