
#include <syb3r/synthesis/ScriptGenerator.h>
#include <iostream>
#include <fstream>

#include <boost/optional.hpp>
#include <string>
#include <vector>

void printUsage(const char *cmd)
{
    std::cout << "Usage: " << cmd << " --filename renderScript.py --outputDir /path/to/output/directory/ "
              << "[--camera cameraLeft] [--camera cameraRight] [--GPU] [--preview] [--preRenderHook preRenderHookScript.py]" << std::endl;
}

int main(int argc, const char **argv)
{

    boost::optional<std::string> filename;
    boost::optional<std::string> outputDirectory;
    std::vector<std::string> cameras;
    bool GPU = false;
    bool preview = false;
    boost::optional<std::string> preRenderHookScriptFilename;

    for (unsigned i = 1; i < argc; i++) {
        std::string argument(argv[i]);

        if (argument == "--filename") {
            if (i+1 < argc) {
                filename = argv[++i];
            } else {
                std::cout << "Missing parameter" << std::endl;
                return -1;
            }
        } else
        if (argument == "--outputDir") {
            if (i+1 < argc) {
                outputDirectory = argv[++i];
            } else {
                std::cout << "Missing parameter" << std::endl;
                return -1;
            }
        } else
        if (argument == "--camera") {
            if (i+1 < argc) {
                cameras.push_back(argv[++i]);
            } else {
                std::cout << "Missing parameter" << std::endl;
                return -1;
            }
        } else
        if (argument == "--GPU") {
            GPU = true;
        } else
        if (argument == "--preview") {
            preview = true;
        } else
        if (argument == "--preRenderHook") {
            if (i+1 < argc) {
                preRenderHookScriptFilename = argv[++i];
            } else {
                std::cout << "Missing parameter" << std::endl;
                return -1;
            }
        } else {
            std::cout << "Unknown argument: " << argument << std::endl;
            return -1;
        }
    }

    if (!filename) {
        std::cout << "Filename for the generated script is mandatory!" << std::endl;
        printUsage(argv[0]);
        return -1;
    }

    if (!outputDirectory) {
        std::cout << "Output directory for the rendered images is mandatory!" << std::endl;
        printUsage(argv[0]);
        return -1;
    }




    syb3r::synthesis::ScripGenerator generator;
    generator.outputDir = *outputDirectory;
    generator.computeDevice = GPU?syb3r::synthesis::ScripGenerator::COMPUTEDEVICE_GPU:syb3r::synthesis::ScripGenerator::COMPUTEDEVICE_CPU;
    generator.cameras = cameras;
    generator.preview = preview;

    if (preRenderHookScriptFilename) {
        std::fstream file;
        file.exceptions( std::fstream::failbit );
        try {
            file.open(preRenderHookScriptFilename->c_str(), std::fstream::in);
            generator.preRenderHook.assign(std::istreambuf_iterator<char>(file), std::istreambuf_iterator<char>());
        } catch (const std::exception &e) {
            std::cout << "Could not read pre render hook script file: " << e.what() << std::endl;
            return -1;
        }
    }

    std::fstream file;
    file.exceptions( std::fstream::failbit );
    try {
        file.open(filename->c_str(), std::fstream::out);
        generator.generate(file);
    } catch (const std::exception &e) {
        std::cout << "Could not write file: " << e.what() << std::endl;
        return -1;
    }

    std::cout << "All done. Execute \"path/to/blender -b path/to/blendfile.blend -P " << *filename << "\" to start rendering!" << std::endl;

    return 0;
}
