#ifndef DATASET_H
#define DATASET_H

#include <Eigen/Dense>
#include <vector>
#include <tinyxml2.h>
#include <iostream>
#include <fstream>

#include <OpenEXR/ImfInputFile.h>
#include <OpenEXR/ImfArray.h>
#include <OpenEXR/ImathBox.h>


#include <boost/optional.hpp>
#include <boost/filesystem.hpp>


#include "Camera.h"
#include "Image.h"

namespace syb3r {
namespace data {

class Dataset
{
    public:
        void loadFromXML(const boost::filesystem::path &xmlFilename);

        void calcPointCloudFromDepth(std::vector<Eigen::Vector3f>& pc, std::vector<unsigned>& mask, unsigned subsamplingStride) const;

        inline const std::vector<Image> &getImages() const { return m_images; }
        inline const boost::optional<float> &getUnitsToMeters() const { return m_unitsToMeters; }

        void addImage(Image image) { m_images.push_back(std::move(image)); }
    protected:
        boost::optional<float> m_unitsToMeters;

        std::vector<Image> m_images;
};

}
}

#endif // DATASET_H
