#ifndef IMAGE_H
#define IMAGE_H

#include "Camera.h"

#include <string>
#include <boost/optional.hpp>
#include <boost/filesystem.hpp>

#include <Eigen/Dense>

#include <cstdint>

namespace tinyxml2 {
    class XMLElement;
}

namespace syb3r {
namespace data {

class Image
{
    public:
        static Image readFromSceneXML(const tinyxml2::XMLElement *imageNode, const boost::filesystem::path &sceneRoot);

        Image(const Camera &camera, const boost::filesystem::path &hdrFilename, const boost::filesystem::path &attribFilename);

        inline const Camera &getCamera() const { return m_camera; }
        inline unsigned getFrameIdx() const { return m_frameIdx; }
        inline const std::string &getCameraName() const { return m_cameraName; }

        inline const boost::filesystem::path &getHDRFilename() const { return m_hdrFilename; }
        inline const boost::filesystem::path &getAttribFilename() const { return m_attribFilename; }

        typedef Eigen::Array<float, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor> FloatArray;
        typedef Eigen::Array<std::uint32_t, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor> UIntArray;
        FloatArray readDistanceMap() const;
        FloatArray readZBuffer(unsigned subsamplingStride = 1) const;
        FloatArray computeZBuffer(const FloatArray &distanceMap, unsigned subsamplingStride = 1) const;
        UIntArray readObjectID() const;

        void extractPointcloud(std::vector<Eigen::Vector3f> &position, std::vector<unsigned> *objectId = nullptr, unsigned subsamplingStride = 1) const;
    protected:
        Camera m_camera;
        unsigned m_frameIdx = ~0u;
        std::string m_cameraName;

        boost::filesystem::path m_hdrFilename;
        boost::filesystem::path m_attribFilename;

        boost::optional<float> m_focalLength;
        boost::optional<float> m_sensorWidth;
        boost::optional<float> m_fNumber;
};

}
}

#endif // IMAGE_H
