#ifndef MVSRECONPOINTCLOUD_H
#define MVSRECONPOINTCLOUD_H

#include <vector>
#include <Eigen/Dense>

namespace syb3r {
namespace data {

/**
 * @brief Stores a point cloud with position, color, normal and sensor visibility information for each point
 */
class Pointcloud
{
    public:
        struct OffsetCount {
            unsigned offset = 0;
            unsigned count = 0;
        };

        void reset(unsigned numPoints);
        void setPoint(unsigned index, const Eigen::Vector3f &position, const Eigen::Vector3f &normal);
        void setPoint(unsigned index, const Eigen::Vector3f &position, const Eigen::Vector3f &normal, const Eigen::Vector3f &color);
        void setPointVisibility(unsigned index, const unsigned *imageIndices, unsigned numImageIndices);


        /// Returns the number of points in the point cloud
        inline unsigned getCount() const { return m_positions.size(); }

        /// Returns the 3D world space position of a point in the point cloud
        inline const Eigen::Vector3f &getPosition(unsigned index) const { return m_positions[index]; }
        /// Returns the 3D world space estimated surface normal at a point in the point cloud
        inline const Eigen::Vector3f &getNormal(unsigned index) const { return m_normals[index]; }
        /// Returns a sRGB [0..1] surface color at a point in the point cloud
        inline const Eigen::Vector3f &getColor(unsigned index) const { return m_colors[index]; }
    protected:
        std::vector<Eigen::Vector3f> m_positions;
        std::vector<Eigen::Vector3f> m_normals;
        std::vector<Eigen::Vector3f> m_colors;
        std::vector<OffsetCount> m_visibilities;
        std::vector<unsigned> m_visibilityImageIndices;
};

}
}

#endif // MVSRECONPOINTCLOUD_H
