#ifndef FRACTIONALICP_H
#define FRACTIONALICP_H

#include <Eigen/Dense>
#include <vector>

namespace syb3r {
namespace math {


class ICPTransform
{
    public:
        virtual void transform(const Eigen::Vector3f *src, Eigen::Vector3f *dst, unsigned count) const = 0;
        virtual void computeLeastSquaresTransform(const Eigen::Vector3f *src, const Eigen::Vector3f *dst, unsigned count) = 0;
};

class ICPTransformHelmert : public ICPTransform
{
    public:
        virtual void transform(const Eigen::Vector3f *src, Eigen::Vector3f *dst, unsigned count) const override;
        virtual void computeLeastSquaresTransform(const Eigen::Vector3f *src, const Eigen::Vector3f *dst, unsigned count) override;

        inline const Eigen::Matrix3f &getRotation() const { return m_rotation; }
        inline float getScale() const { return m_scale; }
        inline const Eigen::Vector3f &getOffset() const { return m_offset; }
    protected:
        Eigen::Matrix3f m_rotation = Eigen::Matrix3f::Identity();
        float m_scale = 1.0f;
        Eigen::Vector3f m_offset = Eigen::Vector3f::Zero();
};


void fractionalICP(const std::vector<Eigen::Vector3f> &src, const std::vector<Eigen::Vector3f> &dst, ICPTransform &transform);
void ICP(const std::vector<Eigen::Vector3f> &src, const std::vector<Eigen::Vector3f> &dst, ICPTransform &transform, float residualPower = 2.0f, float *residual = nullptr);

}
}

#endif // FRACTIONALICP_H
