#include "CameraShake.h"

#include <tinyxml2.h>

namespace syb3r {
namespace models {
    
LinearCameraShake LinearCameraShake::readFromXML(tinyxml2::XMLElement *rootNode)
{
    tinyxml2::XMLElement *node = rootNode->FirstChildElement("LinearCameraShake");
    if (node == nullptr)
        throw std::runtime_error("No LinearCameraShake found!");

    unsigned k;
    float theta;
    
    if (node->QueryUnsignedAttribute("k", &k) != tinyxml2::XML_NO_ERROR)
        throw std::runtime_error("Error parsing values of LinearCameraShake");

    if (node->QueryFloatAttribute("theta", &theta) != tinyxml2::XML_NO_ERROR)
        throw std::runtime_error("Error parsing values of LinearCameraShake");
    
    return LinearCameraShake(k, theta);
}

LinearCameraShake LinearCameraShake::readFromXML(const boost::filesystem::path &filename)
{
    tinyxml2::XMLDocument xmlDoc;
    
    if (xmlDoc.LoadFile(filename.string().c_str()) != tinyxml2::XML_NO_ERROR)
        throw std::runtime_error(std::string("Could not open or parse xml file! (") + filename.string() + ")");

    tinyxml2::XMLElement* pRoot = xmlDoc.FirstChildElement();
    if (pRoot == nullptr) throw std::runtime_error("No root node in xml file!");
    
    return readFromXML(pRoot);    
}

void LinearCameraShake::writeToXML(tinyxml2::XMLElement *rootNode) const
{
    tinyxml2::XMLElement *node = rootNode->GetDocument()->NewElement("LinearCameraShake");
    rootNode->LinkEndChild(node);
    node->SetAttribute("k", m_k);
    node->SetAttribute("theta", m_theta);
}

void LinearCameraShake::writeToXML(const boost::filesystem::path &filename) const
{
    tinyxml2::XMLDocument xmlDoc;

    tinyxml2::XMLElement* pRoot = xmlDoc.NewElement("Camera");
    xmlDoc.InsertEndChild(pRoot);

    writeToXML(pRoot);
    
    if (xmlDoc.SaveFile(filename.string().c_str()) != tinyxml2::XML_NO_ERROR)
        throw std::runtime_error("Could not save scene xml file!");    
}



void LinearCameraShake::computeGammaParametersInPixels(float exposureTime, float imageWidth_pixels, float imageWidth_mm, float focalLength_mm, float &k, float &theta) const
{
    k = m_k;
    theta = m_theta * exposureTime * focalLength_mm * imageWidth_pixels / imageWidth_mm;
}

std::gamma_distribution<float> LinearCameraShake::getMotionBlurInPixels(float exposureTime, float imageWidth_pixels, float imageWidth_mm, float focalLength_mm) const
{
    float k;
    float theta;
    computeGammaParametersInPixels(exposureTime, imageWidth_pixels, imageWidth_mm, focalLength_mm, k, theta);
    return std::gamma_distribution<float>(k, theta);
}


namespace Eos400D_Andy_Standing_noCoffee_linear {
LinearCameraShake cameraShake1_50th(3, 0.00382903f);
LinearCameraShake cameraShake1_10th(4, 0.00821505f);
}

}
}