#ifndef CAMERASHAKE_H
#define CAMERASHAKE_H

#include <Eigen/Dense>
#include <boost/filesystem.hpp>

#include <vector>
#include <string>
#include <initializer_list>

#include <iostream>

#include <random>

namespace tinyxml2 {
    class XMLElement;
}


namespace syb3r {
namespace models {

class LinearCameraShake
{
    public:
        static LinearCameraShake readFromXML(tinyxml2::XMLElement *rootNode); // these only work for extern templates
        static LinearCameraShake readFromXML(const boost::filesystem::path &filename);
        void writeToXML(tinyxml2::XMLElement *rootNode) const;
        void writeToXML(const boost::filesystem::path &filename) const;
        
        
        LinearCameraShake(float k, float theta) : m_k(k), m_theta(theta) { }

        void computeGammaParametersInPixels(float exposureTime, float imageWidth_pixels, float imageWidth_mm, float focalLength_mm, float &k, float &theta) const;
        std::gamma_distribution<float> getMotionBlurInPixels(float exposureTime, float imageWidth_pixels, float imageWidth_mm, float focalLength_mm) const;
    protected:
        float m_k;
        float m_theta;
};

namespace Eos400D_Andy_Standing_noCoffee_linear {
extern LinearCameraShake cameraShake1_50th;
extern LinearCameraShake cameraShake1_10th;

}

}
}

#endif // CAMERASHAKE_H
