#include "SimpleCameraSensorNoise.h"

namespace syb3r {
namespace models {


SimpleCameraSensorNoise::SimpleCameraSensorNoise(float blurSigma, const Eigen::Matrix3f &rgbToComponentVariance, const Eigen::Vector3f &componentVarianceOffset) {
    m_filterKernel.resize(10);
    for (unsigned i = 0; i < m_filterKernel.size(); i++)
        m_filterKernel[i] = 1.0f / (blurSigma * std::sqrt(2.0f * (float)M_PI)) * std::exp(-(float)(i*i)/(2.0f *blurSigma*blurSigma));


    tools::normalizeFilterKernelEnergy(m_filterKernel);
    m_rgbToComponentVariance = rgbToComponentVariance;
    m_componentVarianceOffset = componentVarianceOffset;
}

void SimpleCameraSensorNoise::synthesizeNoise(tools::FloatImage<Eigen::Vector3f> &img, std::mt19937 &rng) const
{
    tools::FloatImage<Eigen::Vector3f> noise;
    noise.resize(img.getWidth(), img.getHeight());
    std::normal_distribution<float> unitNormalDist(0.0f, 1.0f);
    for (unsigned y = 0; y < img.getHeight(); y++)
        for (unsigned x = 0; x < img.getWidth(); x++) {
            Eigen::Vector3f rgb = Eigen::Vector3f(img(x, y)[0], img(x, y)[1], img(x, y)[2]);
            Eigen::Vector3f noiseVar = m_rgbToComponentVariance * rgb + m_componentVarianceOffset;
            noiseVar[0] = std::max(noiseVar[0], 0.0f);
            noiseVar[1] = std::max(noiseVar[1], 0.0f);
            noiseVar[2] = std::max(noiseVar[2], 0.0f);

            noise(x, y)[0] = unitNormalDist(rng) * std::sqrt(noiseVar[0]);
            noise(x, y)[1] = unitNormalDist(rng) * std::sqrt(noiseVar[1]);
            noise(x, y)[2] = unitNormalDist(rng) * std::sqrt(noiseVar[2]);
        }

    tools::FloatImage<Eigen::Vector3f> tmp;
    tools::blurHorizontal(tmp, noise, m_filterKernel);
    tools::blurVertical(noise, tmp, m_filterKernel);

    img += noise;
}


SimpleCameraSensorNoise eos400_1600(
    0.745f,
    (Eigen::Matrix3f() << 
        0.00537437f, -0.000402357f, -8.4267e-05f, 
        -0.000160993f, 0.00250866f, 0.000129777f,
        2.5017e-05f, -7.92174e-05f, 0.00379866f
    ).finished(),
    Eigen::Vector3f(
        0.000217488f, -0.000131628f, 0.000186254f
    )
);

}
}