/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "FloatImage.h"

namespace syb3r {
namespace tools {

template class FloatImage<Eigen::Vector3f>;
template class FloatImage<float>;

void normalizeFilterKernel(std::vector<float> &kernel)
{
    float sum = kernel[0];
    for (unsigned i = 1; i < kernel.size(); i++)
        sum += 2.0f * kernel[i];
    float fac = 1.0f / sum;
    for (unsigned i = 0; i < kernel.size(); i++)
        kernel[i] *= fac;
}

void normalizeFilterKernelEnergy(std::vector<float> &kernel)
{
    float sum = kernel[0]*kernel[0];
    for (unsigned i = 1; i < kernel.size(); i++)
        sum += 2.0f * kernel[i]*kernel[i];
    float fac = 1.0f / std::sqrt(sum);
    for (unsigned i = 0; i < kernel.size(); i++)
        kernel[i] *= fac;
}


}
}