#ifndef TMPDIR_H
#define TMPDIR_H

#include <boost/filesystem.hpp>

namespace syb3r {
namespace tools {

/**
 * @brief Creates a temporary directory in a suitable location that is deleted when the class instance is destroyed
 * @details The directory is created as a subdirectory of the temp folder (usually /tmp/).
 */
class TmpDir
{
    public:
        TmpDir();
        ~TmpDir();
        TmpDir(const TmpDir&) = delete;
        const TmpDir& operator=(const TmpDir&) = delete;

        /// Returns the path to the temporary directory
        inline const boost::filesystem::path &getPath() const { return m_path; }
    protected:
        boost::filesystem::path m_path;
};

}
}

#endif // TMPDIR_H
