#include "WrapperVSFM.h"

#include <boost/filesystem.hpp>

#include <sstream>
#include <fstream>
#include <iostream>

#include <syb3r/tools/TmpDir.h>
#include <syb3r/data/EstimatedCameraPath.h>
#include <syb3r/data/Dataset.h>

namespace syb3r {
namespace wrapper {

bool WrapperVSFM::runSFM(const data::Dataset &dataset, const std::vector<std::string> &images, data::EstimatedCameraPath &estimatedCameraPath)
{
    std::stringstream command;

    tools::TmpDir tmpDir;
    std::cout << "Creating image symlinks in " << tmpDir.getPath().string() << std::endl;

    for (const std::string &img : images)
        if (hardlink)
            boost::filesystem::create_hard_link(img, tmpDir.getPath()/boost::filesystem::path(img).filename());
        else
            boost::filesystem::copy(img, tmpDir.getPath()/boost::filesystem::path(img).filename());

    command << "export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:" << executablePath << " && " << executablePath<<executable;

    command << " sfm+shared+sort";
    if (producePMVSOutput)
        command << "+cmvs";
    command << " " << tmpDir.getPath().string() << " "  << nvmOutputFilename;

    bool success = run(command.str().c_str());

    if (success)
        parseCamerasFromNVMFile(dataset, images, nvmOutputFilename, estimatedCameraPath);

    return success;
}

void WrapperVSFM::parseLine(const char *line)
{
    std::cout << "SFM: " << line;
}

void WrapperVSFM::parseCamerasFromNVMFile(const data::Dataset &dataset, const std::vector<std::string> &images, const std::string &nvmOutputFilename, data::EstimatedCameraPath &estimatedCameraPath)
{
    std::ifstream inputStream;
    inputStream.exceptions(std::ifstream::failbit | std::ifstream::badbit);
    inputStream.open(nvmOutputFilename, std::fstream::in);

    std::string line;
    std::getline(inputStream, line);
    if (line != "NVM_V3 ")
        throw std::runtime_error("Invalid file!");

    unsigned numCameras;
    inputStream >> numCameras;
    std::getline(inputStream, line);
    for (unsigned i = 0; i < numCameras; i++) {
        std::getline(inputStream, line);
        std::stringstream lineStream(line);

        std::string filename;
        std::getline(lineStream, filename, '\t');

        std::string filenameWOPath = boost::filesystem::path(filename).filename().string();
        unsigned srcIndex = ~0u;
        for (unsigned j = 0; j < images.size(); j++)
            if (filenameWOPath == boost::filesystem::path(images[j]).filename().string()) {
                srcIndex = j;
                break;
            }

        if (srcIndex == ~0u)
            throw std::runtime_error("One nvm image filename does not match any of the files given to vsfm...!");

        float focalLength;
        lineStream >> focalLength;

        Eigen::Quaternionf rotation;
        lineStream >> rotation.w() >> rotation.x() >> rotation.y() >> rotation.z();

        Eigen::Vector3f focalPoint;
        lineStream >> focalPoint.x() >> focalPoint.y() >> focalPoint.z();

        float radialDistortion;
        lineStream >> radialDistortion;

        float shouldBeZero;
        lineStream >> shouldBeZero;

        if (shouldBeZero != 0.0f)
            throw std::runtime_error("Parsing error, the last number in a nvm camera line should be zero!");

        data::Camera camera = data::Camera::convertFrom_VSFM_NVM(focalLength, rotation, focalPoint, dataset.getImages()[i].getCamera().getWidth(), dataset.getImages()[i].getCamera().getHeight());
        estimatedCameraPath.add(camera, srcIndex);
    }
}


}
}