/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_AutoExposure.h"

#include "ImagePostprocessor.h"

namespace syb3r {
namespace synthesis {

ImagePP_AutoExposure::ImagePP_AutoExposure(bool apply) : m_apply(apply)
{

}

void ImagePP_AutoExposure::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &properties) const
{
    float avgBrightnessSum = 0.0f;
    float avgBrightnessDenom = 0.0f;
    for (unsigned y = 0; y < input.getHeight(); y++) {
        for (unsigned x = 0; x < input.getWidth(); x++) {
            float dx = x - input.getWidth() * 0.5f;
            float dy = y - input.getHeight() * 0.5f;
            float weight = std::max(0.0f, 1.0f - ((dx*dx) + (dy*dy)) / (input.getHeight()*input.getHeight()*0.125f));

            float r = input(x, y)[0];
            float g = input(x, y)[1];
            float b = input(x, y)[2];

            float lum = (r+g+b) * 0.3333f;

            avgBrightnessSum += lum * weight;
            avgBrightnessDenom += weight;
        }
    }

    float factor = avgBrightnessDenom / avgBrightnessSum * 0.4f;
    properties.set("autoExposureCorrectionFactor", factor);

    if (m_apply) {
        output.resize(input.getWidth(), input.getHeight());
        for (unsigned y = 0; y < input.getHeight(); y++)
            for (unsigned x = 0; x < input.getWidth(); x++)
                output(x, y) = input(x, y) * factor;
    } else {
        output = input;
    }
}

}
}