/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_ColorMatrix.h"

namespace syb3r {
namespace synthesis {

const Eigen::Matrix3f ImagePP_ColorMatrix::CanonEOS400_to_XYZ((Eigen::Matrix3f() <<
    1.60727f,   0.14302f,   0.13396f,
    0.82267f,   0.73836f,  -0.17135f,
    0.11427f,  -0.11048f,   1.33575f
).finished());

const Eigen::Matrix3f ImagePP_ColorMatrix::XYZ_to_sRGB((Eigen::Matrix3f() <<
    3.240513f,  -1.537166f,  -0.498541f,
   -0.969264f,   1.875994f,   0.041557f,
    0.055647f,  -0.204042f,   1.057311f
).finished());


ImagePP_ColorMatrix::ImagePP_ColorMatrix(const Eigen::Matrix3f &colorMatrix) : m_colorMatrix(colorMatrix)
{
}

void ImagePP_ColorMatrix::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &) const
{
    output.resize(input.getWidth(), input.getHeight());
    for (unsigned i = 0; i < input.getWidth()*input.getHeight(); i++)
        output[i] = m_colorMatrix * input[i];
}

}
}
