/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_DemosaicSimple.h"

#include <stdexcept>

namespace syb3r {
namespace synthesis {

ImagePP_DemosaicSimple::ImagePP_DemosaicSimple()
{

}

void ImagePP_DemosaicSimple::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &) const
{
    if (input.getWidth() % 4 != 0)
        throw std::runtime_error("Width must be divisible by 4!");

    if (input.getHeight() % 4 != 0)
        throw std::runtime_error("Height must be divisible by 4!");

    output.resize(input.getWidth(), input.getHeight());
    for (unsigned y = 2; y < input.getHeight()-2; y+=2) {
        for (unsigned x = 2; x < input.getWidth()-2; x+=2) {
            output(x+0, y+0)[0] = input(x+0, y+0)[0];
            output(x+0, y+0)[1] = (input(x-1, y+0)[1] + input(x+1, y+0)[1] + input(x+0, y-1)[1] + input(x+0, y+1)[1]) * 0.25f;
            output(x+0, y+0)[2] = (input(x-1, y-1)[2] + input(x+1, y-1)[2] + input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.25f;

            output(x+1, y+0)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
            output(x+1, y+0)[1] = input(x+1, y+0)[1];
            output(x+1, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+0, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
            output(x+0, y+1)[1] = input(x+0, y+1)[1];
            output(x+0, y+1)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0] + input(x+0, y+2)[0] + input(x+2, y+2)[0]) * 0.25f;
            output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+1, y+2)[1] + input(x+2, y+1)[1]) * 0.25f;
            output(x+1, y+1)[2] = input(x+1, y+1)[2];
        }
    }
    for (unsigned x = 2; x < input.getWidth()-2; x+=2) {
        {
            unsigned y = 0;
            output(x+0, y+0)[0] = input(x+0, y+0)[0];
            output(x+0, y+0)[1] = (input(x-1, y+0)[1] + input(x+1, y+0)[1] + input(x+0, y+1)[1]) * 0.333f;
            output(x+0, y+0)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+1, y+0)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
            output(x+1, y+0)[1] = input(x+1, y+0)[1];
            output(x+1, y+0)[2] = input(x+1, y+1)[2];

            output(x+0, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
            output(x+0, y+1)[1] = input(x+0, y+1)[1];
            output(x+0, y+1)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0] + input(x+0, y+2)[0] + input(x+2, y+2)[0]) * 0.25f;
            output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+1, y+2)[1] + input(x+2, y+1)[1]) * 0.25f;
            output(x+1, y+1)[2] = input(x+1, y+1)[2];
        }
        {
            unsigned y = input.getHeight()-2;
            output(x+0, y+0)[0] = input(x+0, y+0)[0];
            output(x+0, y+0)[1] = (input(x-1, y+0)[1] + input(x+1, y+0)[1] + input(x+0, y-1)[1] + input(x+0, y+1)[1]) * 0.25f;
            output(x+0, y+0)[2] = (input(x-1, y-1)[2] + input(x+1, y-1)[2] + input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.25f;

            output(x+1, y+0)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
            output(x+1, y+0)[1] = input(x+1, y+0)[1];
            output(x+1, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+0, y+1)[0] = input(x+0, y+0)[0];
            output(x+0, y+1)[1] = input(x+0, y+1)[1];
            output(x+0, y+1)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
            output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+2, y+1)[1]) * 0.333f;
            output(x+1, y+1)[2] = input(x+1, y+1)[2];
        }
    }
    for (unsigned y = 2; y < input.getHeight()-2; y+=2) {
        {
            unsigned x = 0;
            output(x+0, y+0)[0] = input(x+0, y+0)[0];
            output(x+0, y+0)[1] = (input(x+1, y+0)[1] + input(x+0, y-1)[1] + input(x+0, y+1)[1]) * 0.333f;
            output(x+0, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+1, y+0)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
            output(x+1, y+0)[1] = input(x+1, y+0)[1];
            output(x+1, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+0, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
            output(x+0, y+1)[1] = input(x+0, y+1)[1];
            output(x+0, y+1)[2] = input(x+1, y+1)[2];

            output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0] + input(x+0, y+2)[0] + input(x+2, y+2)[0]) * 0.25f;
            output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+1, y+2)[1] + input(x+2, y+1)[1]) * 0.25f;
            output(x+1, y+1)[2] = input(x+1, y+1)[2];
        }
        {
            unsigned x = input.getWidth()-2;
            output(x+0, y+0)[0] = input(x+0, y+0)[0];
            output(x+0, y+0)[1] = (input(x-1, y+0)[1] + input(x+1, y+0)[1] + input(x+0, y-1)[1] + input(x+0, y+1)[1]) * 0.25f;
            output(x+0, y+0)[2] = (input(x-1, y-1)[2] + input(x+1, y-1)[2] + input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.25f;

            output(x+1, y+0)[0] = input(x+0, y+0)[0];
            output(x+1, y+0)[1] = input(x+1, y+0)[1];
            output(x+1, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+0, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
            output(x+0, y+1)[1] = input(x+0, y+1)[1];
            output(x+0, y+1)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

            output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
            output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+1, y+2)[1]) * 0.333f;
            output(x+1, y+1)[2] = input(x+1, y+1)[2];
        }
    }
    {
        unsigned x = 0;
        unsigned y = 0;
        output(x+0, y+0)[0] = input(x+0, y+0)[0];
        output(x+0, y+0)[1] = (input(x+1, y+0)[1] + input(x+0, y+1)[1]) * 0.5f;
        output(x+0, y+0)[2] = input(x+1, y+1)[2];

        output(x+1, y+0)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
        output(x+1, y+0)[1] = input(x+1, y+0)[1];
        output(x+1, y+0)[2] = input(x+1, y+1)[2];

        output(x+0, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
        output(x+0, y+1)[1] = input(x+0, y+1)[1];
        output(x+0, y+1)[2] = input(x+1, y+1)[2];

        output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0] + input(x+0, y+2)[0] + input(x+2, y+2)[0]) * 0.25f;
        output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+1, y+2)[1] + input(x+2, y+1)[1]) * 0.25f;
        output(x+1, y+1)[2] = input(x+1, y+1)[2];
    }
    {
        unsigned x = input.getWidth()-2;
        unsigned y = 0;
        output(x+0, y+0)[0] = input(x+0, y+0)[0];
        output(x+0, y+0)[1] = (input(x-1, y+0)[1] + input(x+1, y+0)[1] + input(x+0, y+1)[1]) * 0.333f;
        output(x+0, y+0)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

        output(x+1, y+0)[0] = input(x+0, y+0)[0];
        output(x+1, y+0)[1] = input(x+1, y+0)[1];
        output(x+1, y+0)[2] = input(x+1, y+1)[2];

        output(x+0, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
        output(x+0, y+1)[1] = input(x+0, y+1)[1];
        output(x+0, y+1)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

        output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+0, y+2)[0]) * 0.5f;
        output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+1, y+2)[1]) * 0.333f;
        output(x+1, y+1)[2] = input(x+1, y+1)[2];
    }
    {
        unsigned x = 0;
        unsigned y = input.getHeight()-2;
        output(x+0, y+0)[0] = input(x+0, y+0)[0];
        output(x+0, y+0)[1] = (input(x+1, y+0)[1] + input(x+0, y-1)[1] + input(x+0, y+1)[1]) * 0.333f;
        output(x+0, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

        output(x+1, y+0)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
        output(x+1, y+0)[1] = input(x+1, y+0)[1];
        output(x+1, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

        output(x+0, y+1)[0] = input(x+0, y+0)[0];
        output(x+0, y+1)[1] = input(x+0, y+1)[1];
        output(x+0, y+1)[2] = input(x+1, y+1)[2];

        output(x+1, y+1)[0] = (input(x+0, y+0)[0] + input(x+2, y+0)[0]) * 0.5f;
        output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1] + input(x+2, y+1)[1]) * 0.333f;
        output(x+1, y+1)[2] = input(x+1, y+1)[2];
    }
    {
        unsigned x = input.getWidth()-2;
        unsigned y = input.getHeight()-2;

        output(x+0, y+0)[0] = input(x+0, y+0)[0];
        output(x+0, y+0)[1] = (input(x-1, y+0)[1] + input(x+1, y+0)[1] + input(x+0, y-1)[1] + input(x+0, y+1)[1]) * 0.25f;
        output(x+0, y+0)[2] = (input(x-1, y-1)[2] + input(x+1, y-1)[2] + input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.25f;

        output(x+1, y+0)[0] = input(x+0, y+0)[0];
        output(x+1, y+0)[1] = input(x+1, y+0)[1];
        output(x+1, y+0)[2] = (input(x+1, y-1)[2] + input(x+1, y+1)[2]) * 0.5f;

        output(x+0, y+1)[0] = input(x+0, y+0)[0];
        output(x+0, y+1)[1] = input(x+0, y+1)[1];
        output(x+0, y+1)[2] = (input(x-1, y+1)[2] + input(x+1, y+1)[2]) * 0.5f;

        output(x+1, y+1)[0] = input(x+0, y+0)[0];
        output(x+1, y+1)[1] = (input(x+0, y+1)[1] + input(x+1, y+0)[1]) * 0.5f;
        output(x+1, y+1)[2] = input(x+1, y+1)[2];
    }
}

}
}