/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef IMAGEPP_LINEARBLUR_H
#define IMAGEPP_LINEARBLUR_H

#include "ImagePostprocessorStep.h"

#include <syb3r/models/CameraShake.h>

#include <mutex>
#include <random>
#include <map>

namespace syb3r {
namespace synthesis {


class ImagePP_LinearBlur : public ImagePostprocessorStep
{
    public:
        std::map<float, syb3r::models::LinearCameraShake> exposureTimeToCameraShake;

        ImagePP_LinearBlur();

        virtual void process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &properties) const override;
    protected:
        mutable std::mutex m_rngMutex;
        mutable std::mt19937 m_rng;
};

}
}


#endif // IMAGEPP_LINEARBLUR_H
