/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_LinearToGamma.h"

namespace syb3r {
namespace synthesis {

ImagePP_LinearToGamma::ImagePP_LinearToGamma()
{
}

void ImagePP_LinearToGamma::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &) const
{
    output.resize(input.getWidth(), input.getHeight());
    for (unsigned i = 0; i < input.getWidth() * input.getHeight(); i++) {
        for (unsigned c = 0; c < 3; c++) {
            float v = std::min<float>(std::max<float>(input[i][c], 0.0f), 1.0f);
            if (v <= 0.0031308f)
                output[i][c] = 12.92f * v;
            else
                output[i][c] = (1+0.055f)*std::pow(v, 1.0f / 2.4f) - 0.055f;
        }
    }
}

}
}