/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_OldSensorNoise.h"

#include "ImagePostprocessor.h"

namespace syb3r {
namespace synthesis {

ImagePP_OldSensorNoise::ImagePP_OldSensorNoise()
{
    isoToSensorNoise.insert(std::make_pair(1600, models::eos400_1600));
}

void ImagePP_OldSensorNoise::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &properties) const
{
    std::mt19937 threadLocalRng;
    {
        std::lock_guard<std::mutex> lock(m_rngMutex);
        threadLocalRng.seed(m_rng());
    }

    int iso = properties.getInt("iso");

    if (iso != 0) {
        output.resize(input.getWidth(), input.getHeight());

        auto it = isoToSensorNoise.find(iso);
        if (it != isoToSensorNoise.end()) {
            output = input;
            it->second.synthesizeNoise(output, threadLocalRng);        
        } else 
            throw std::runtime_error("Iso value for which no noise model was registered!");
    } else
        output = input;
}

}
}