/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_SensorNoise.h"

#include "ImagePostprocessor.h"

namespace syb3r {
namespace synthesis {

ImagePP_SensorNoise::ImagePP_SensorNoise(float sigmaNormalNoise, float sigmaLognormalNoise) :
        m_sigmaNormalNoise(sigmaNormalNoise), m_sigmaLognormalNoise(sigmaLognormalNoise)
{

}

void ImagePP_SensorNoise::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &properties) const
{
    std::mt19937 threadLocalRng;
    {
        std::lock_guard<std::mutex> lock(m_rngMutex);
        threadLocalRng.seed(m_rng());
    }

    std::normal_distribution<float> additiveNoise(0.0f, m_sigmaNormalNoise);
    std::lognormal_distribution<float> multiplicativeNoise(0.0f, m_sigmaLognormalNoise);

    output.resize(input.getWidth(), input.getHeight());
    for (unsigned i = 0; i < input.getWidth()*input.getHeight(); i++) {
        output[i][0] = input[i][0] * multiplicativeNoise(threadLocalRng) + additiveNoise(threadLocalRng);
        output[i][1] = input[i][1] * multiplicativeNoise(threadLocalRng) + additiveNoise(threadLocalRng);
        output[i][2] = input[i][2] * multiplicativeNoise(threadLocalRng) + additiveNoise(threadLocalRng);
    }
}

}
}