/*
 * Syb3r - A framework for the creation and use of synthetic benchmark datasets for SfM/MVS with Blender and Cycles.
 * Copyright (C) 2016 Andreas Ley <mail@andreas-ley.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ImagePP_TonemappingCurve.h"

namespace syb3r {
namespace synthesis {

ImagePP_TonemappingCurve::ImagePP_TonemappingCurve(const models::CameraRGBResponseCurves<256> &curves) :
        m_curves(curves)
{
}

void ImagePP_TonemappingCurve::process(tools::FloatImage<Eigen::Vector3f> &output, const tools::FloatImage<Eigen::Vector3f> &input, ImageProperties &) const
{
    output.resize(input.getWidth(), input.getHeight());
    for (unsigned i = 0; i < input.getWidth() * input.getHeight(); i++) {
        output[i][0] = m_curves.r.ldrFromHdr(input[i][0]) / 256.0f;
        output[i][1] = m_curves.g.ldrFromHdr(input[i][1]) / 256.0f;
        output[i][2] = m_curves.b.ldrFromHdr(input[i][2]) / 256.0f;
    }
}

}
}