#include <boost/test/unit_test.hpp>

#include <boost/test/floating_point_comparison.hpp>

#include <syb3r/math/HelmertTransformation.h>

#include <random>

#include <iomanip>
#include <iostream>


BOOST_AUTO_TEST_CASE(testHelmertTransformation)
{
    std::default_random_engine generator(133700);
    std::uniform_real_distribution<float> distribution(-1.0f, 1.0f);


    for (unsigned iter = 0; iter < 10; iter++) {
        std::vector<Eigen::Vector3f> points;
        points.resize(100);
        for (auto &p : points) {
            p[0] = distribution(generator) * 10.0f;
            p[1] = distribution(generator) * 10.0f;
            p[2] = distribution(generator) * 10.0f;
        }

        Eigen::Vector3f shift(distribution(generator), distribution(generator), distribution(generator));
        shift *= 10.0f;

        float scale = 0.1f + (distribution(generator) + 1.0f);

        Eigen::Matrix3f rotation = Eigen::AngleAxis<float>(distribution(generator) * M_PI * 2.0f, Eigen::Vector3f(1.0f, 0.0f, 0.0f)) *
                                   Eigen::AngleAxis<float>(distribution(generator) * M_PI * 2.0f, Eigen::Vector3f(0.0f, 1.0f, 0.0f)) *
                                   Eigen::AngleAxis<float>(distribution(generator) * M_PI * 2.0f, Eigen::Vector3f(0.0f, 0.0f, 1.0f)).matrix();

        std::vector<Eigen::Vector3f> transformedPoints;
        transformedPoints.resize(points.size());
        for (unsigned i = 0; i < points.size(); i++) {
            transformedPoints[i] = rotation * points[i] * scale + shift;
            transformedPoints[i][0] += distribution(generator) * 0.0001f;
            transformedPoints[i][1] += distribution(generator) * 0.0001f;
            transformedPoints[i][2] += distribution(generator) * 0.0001f;
        }

        Eigen::Matrix4f transformation = syb3r::math::computeHelmertTransformation<float>(points.data(), transformedPoints.data(), points.size());

        BOOST_CHECK(transformation(3, 0) == 0.0f);
        BOOST_CHECK(transformation(3, 1) == 0.0f);
        BOOST_CHECK(transformation(3, 2) == 0.0f);
        BOOST_CHECK(transformation(3, 3) == 1.0f);

        for (unsigned i = 0; i < points.size(); i++) {
            Eigen::Vector4f p = transformation * Eigen::Vector4f(points[i][0], points[i][1], points[i][2], 1.0f);

            //std::cout << transformedPoints[i] << std::endl << p << std::endl << std::endl;

            BOOST_CHECK(std::abs(transformedPoints[i][0] - p[0]) < 1e-3f);
            BOOST_CHECK(std::abs(transformedPoints[i][1] - p[1]) < 1e-3f);
            BOOST_CHECK(std::abs(transformedPoints[i][2] - p[2]) < 1e-3f);
        }
    }
}